var cv = document.getElementsByTagName("canvas")[0];
var ctx = cv.getContext('2d');
var w = cv.width;
var h = cv.height;

ctx.webkitImageSmoothingEnabled = false;
ctx.mozImageSmoothingEnabled = false;
ctx.imageSmoothingEnabled = false;

var gravity = 150;
var cloudSpeed = 10;
var img1 = document.getElementById('offline-resources-1x');
var img2 = document.getElementById('offline-resources-1x-2');

var now = (new Date()).getTime() / 1000.0;
var lastTime = (new Date()).getTime() / 1000.0;
var delta = 0;

function Sprite(x, y, width, height, img = img1) {
	this.x = x;
	this.y = y;
	this.width = width;
	this.height = height;
	this.img = img;
	this.draw = function(cx, cy, scale = 1) {
		ctx.drawImage(this.img, this.x, this.y, this.width, this.height, cx, cy, this.width * scale, this.height * scale);
	};
}

var rains = [];

function Rain(x, y, cloud) {
	this.x = x;
	this.y = y;
	this.height = 0;
	this.isRaining = true;
	this.cloud = cloud;
	
	this.draw = function() {
		ctx.save();
		var rainHeight = 53;
		var ox = (now * 100) % rainHeight;
		ctx.translate(0, ox);
		ctx.fillStyle = rain;
		ctx.fillRect(this.x, -ox + this.y, 41, this.height);
		ctx.restore();
		
		if(this.x + 41 > dinoX && this.x < dinoX + dino_idle.width
			&& this.y < h - 47 && this.y + this.height > h - 47) {
			dinoStats.thirst += delta * 0.2;
			if(dinoStats.thirst >= 1.5 && dinoState != DINO_STATES.dead) {
				dinoState = DINO_STATES.dead;
				
				deathReason = sel(["he drowned", "you drowned a dinosaur.. in a desert"]);
			} else if(dinoState != DINO_STATES.dead) {				
				dinoState = DINO_STATES.drinking;
				nextDecision = now + 0.5;
			}
		}
		
		if(this.y < h - 8 && this.y + this.height > h - 8) {
			for(var i = 0; i < trees.length; i++) {
				var tree = trees[i];
				if(Math.abs(tree.x - (this.x + 20)) < 10) {
					tree.stage += delta * 0.7;
				}
			}
		
			if(Math.random() < 0.25) {
				grass.push(new Grass(this.x + parseInt(Math.random() * 41) - 8, h - parseInt(Math.random() * 4) - 6));
			}
			
			if(Math.random() < 0.5) {
				var overCrowded = false;
				var treeX = this.x + Math.random() * 41;
				for(var i = 0; i < trees.length; i++) {
					if(Math.abs((trees[i].x) - treeX) < 54) {
						overCrowded = true;
					}
				}
				
				if(!overCrowded) {
					trees.push(new Tree(treeX, h + parseInt(Math.random() * 8)));
				}
			} else if(Math.random() < 0.25) {
				var overCrowded = false;
				var flowerX = this.x + parseInt(Math.random() * 41) - 8;
				for(var i = 0; i < flowers.length; i++) {
					if(Math.abs((flowers[i].x) - flowerX) < 8) {
						overCrowded = true;
					}
				}
				
				if(!overCrowded) {
					flowers.push(new Flower(flowerX, h - 20 - parseInt(Math.random() * 8)));
				}
			}
		}
		
		if(!this.isRaining) {
			this.y += delta * 100;
		} else {
			this.height += delta * 100;
			this.height = Math.min(this.height, h - this.y - 4);
		}
	}
}

var ground = new Sprite(0, 53, 1233, 15);
var cloud = new Sprite(86, 2, 46, 13);

var cloudOffsets = [[0, 0], [100, -20], [261, 15], [-75, 8], [384, 15]];

var dino_idle = new Sprite(848, 2, 44, 47);
var dino_blink = new Sprite(892, 2, 44, 47);
var dino_step1 = new Sprite(936, 2, 44, 47);
var dino_step2 = new Sprite(980, 2, 44, 47);
var dino_woah = new Sprite(1024, 2, 44, 47);
var dino_crouch1 = new Sprite(1112, 2, 59, 47);
var dino_crouch2 = new Sprite(1171, 2, 59, 47);
var dino_dead = new Sprite(24, 0, 40, 43, img2);

var fat0 = new Sprite(49, 44, 44, 47, img2);
var fat1 = new Sprite(100, 44, 44, 47, img2);
var fat2 = new Sprite(151, 44, 44, 47, img2);

var bird0 = new Sprite(134, 2, 46, 40);
var bird1 = new Sprite(180, 2, 46, 40);
var bird_ded = new Sprite(0, 45, 46, 34, img2);
var bird_bones = new Sprite(64, 0, 42, 35, img2);

var rock = new Sprite(544, 2, 40, 40);

var emote_hungry = new Sprite(0, 0, 24, 33, img2);
var emote_content = new Sprite(0, 91, 24, 33, img2);
var emote_full = new Sprite(26, 91, 24, 33, img2);
var emote_thirsty = new Sprite(52, 91, 24, 33, img2);

var grassSprite = new Sprite(227, 89, 29, 8, img2);
var grass = [];
function Grass(x, y) {
	this.x = x;
	this.y = y;
	
	this.draw = function() {
		grassSprite.draw(this.x, this.y);
	}
}

var flowerSprites = [ new Sprite(236, 0, 20, 26, img2), new Sprite(236, 26, 20, 29, img2), new Sprite(236, 56, 20, 28, img2) ];
var flowers = [];

function Flower(x, y) {
	this.x = x;
	this.y = y;
	
	this.sprite = sel(flowerSprites);
	
	this.draw = function() {
		this.sprite.draw(this.x, this.y);
	}
}

var treeSprites = [ new Sprite(4, 238, 13, 18, img2), new Sprite(28, 232, 15, 24, img2), new Sprite(47, 226, 18, 30, img2),
					new Sprite(66, 206, 30, 50, img2), new Sprite(100, 186, 53, 70, img2) ];
var trees = [];

function Tree(x, y) {
	this.x = x;
	this.y = y;
	
	this.stage = 0;
	
	this.draw = function() {
		var s = treeSprites[Math.min(treeSprites.length - 1, parseInt(this.stage))];
		s.draw(parseInt(this.x - (s.width / 2)), this.y - (s.height * 2), 2);
	}
}

var rain = ctx.createPattern(document.getElementById('rain'), 'repeat');

function dist(a, b) {
	return Math.sqrt(((a.x - b.x) * (a.x - b.x)) + ((a.y - b.y) * (a.y - b.y)));
}

function sel(ar) {
	return ar[parseInt(Math.random() * ar.length)];
}

var lightnings = [];

function Lightning(startX, startY, endX, endY) {
	var fragments = 5.0;
	
	var incX = (endX - startX) / fragments;
	var incY = (endY - startY) / fragments;
	
	this.age = 0;
	this.points = [];
	
	this.points.push({ x: startX, y: startY });
	
	for(var i = 1; i < fragments; i++) {
		var randX = Math.random() * 20 - 10;
		var randY = Math.random() * 20 - 10;
		this.points.push({x: startX + (i * incX) + randX, y: startY + (i * incY) + randY});
	}
	
	this.points.push({ x: endX, y: endY });
	
	for(var i = 0; i < this.points.length; i++) {
		if(dist({x: dinoX + (dino_idle.width / 2), y: h - 47 + (dino_idle.height / 2) }, this.points[i]) < 20 && dinoState != DINO_STATES.dead) {
			dinoState = DINO_STATES.dead;
			
			deathReason = sel(["he got smote", "how could you", "what did dino ever do to you??", "lol", "prankd", "rekt", "you had one job", "i guess dinosaurs and electricity dont mix", "whatd you expect", "git gud", "gg ez pz", "im shocked", "shocking"]);
		}
	}
	
	this.draw = function() {
		this.age += delta;
		if(this.age >= 0.2) {
			lightnings.splice(lightnings.indexOf(this), 1);
		}
		
		if(now % 0.02 < 0.01) {
			ctx.save();
			ctx.strokeStyle = '#DADADA';
			ctx.lineWidth = 2;
			ctx.beginPath();
			ctx.moveTo(this.points[0].x, this.points[0].y);
			for(var i = 1; i < this.points.length; i++)
				ctx.lineTo(this.points[i].x, this.points[i].y);
			ctx.stroke();
			ctx.restore();
		}
	};
}

function Bird(x, y, forward) {
	this.x = x;
	this.y = y;
	this.vy = 0;
	this.dead = false;
	this.eaten = 0;
	this.forward = forward;
	this.draw = function() {
		if(this.eaten >= 1) {
			bird_bones.draw(this.x, this.y);
		} else if(this.dead == true) {
			this.vy += gravity * delta;
			this.y += this.vy * delta;
			
			if(this.y >= h - 29) {
				this.y = h - 29;
				this.vy = 0;
				
				if(dinoState != DINO_STATES.dead) {
					dinoState = DINO_STATES.eating;
				}
			}
			
			bird_ded.draw(this.x, this.y);
		} else {
			this.x += delta * 150 * (this.forward ? 1 : -1);
		
			if(this.forward) {
				ctx.save();
				ctx.scale(-1, 1);
			}
			(now % 0.3 < 0.15 ? bird0 : bird1).draw(this.forward ? -bird0.width - this.x : this.x, this.y);
			if(this.forward) {
				ctx.restore();
			}
		}
		
		if(this.x < -bird0.width || this.y > w) {
			birds.splice(birds.indexOf(this), 1);
		}
	};
}

function Rock(x, y, vx, vy) {
	this.x = x;
	this.y = y;
	this.vx = vx;
	this.vy = vy;
	this.draw = function() {
		this.vy += gravity * delta;
		this.x += this.vx * delta;
		this.y += this.vy * delta;
		rock.draw(this.x, this.y);
		var center = {x: this.x + 20, y: this.y + 20};
		if(dist({x: dinoX + (dino_idle.width / 2), y: h - 47 + (dino_idle.height / 2) }, center) < 30 && dinoState != DINO_STATES.dead) {
			dinoState = DINO_STATES.dead;
			
			deathReason = sel(["you struck him with a meteor? ...why?", "how could you", "what did dino ever do to you??", "lol", "you had one job", "i guess dinosaurs and meteors dont mix", "whatd you expect", "ur aim sux", "git gud", "gg ez pz"]);
		}
		
		for(var i = 0; i < birds.length; i++) {
			if(!birds[i].dead && dist({x: birds[i].x + (bird0.width / 2), y: birds[i].y + (bird0.height / 2) }, center) < 30) {
				birds[i].dead = true;
			}
		}
	}
}

var MOUSE_STATES = {
	none: 0,
	meteor: 1,
	rain: 2,
	lightning: 3
};

var mouseState = MOUSE_STATES.none;
var mouseX = 0;
var mouseY = 0;

var DINO_STATES = {
	idle: 0,
	walking: 1,
	emote: 2,
	dead: 3,
	eating: 4,
	drinking: 5
};

var dinoStats = {
	hunger: 0.5,
	thirst: 0.7,
	weight: 0
};

var dinoState = DINO_STATES.idle;
var dinoForward = true;

var dinoSpeed = 100;
var dinoX = (w / 2) - dino_idle.width;

var nextDecision = 0;

var birds = [];
var rocks = [];

var deathReason = "idk what u did this time";

function spawnBird() {
	var left = Math.random() < 0.5;
	birds.push(new Bird(left ? -bird0.width : this.w + bird0.width, h / 2 - (Math.random() * 100) - 20, left));
	setTimeout(spawnBird, Math.random() * (10 - Math.max(0, Math.min((dinoStats.hunger - 0.5) * 7, 9))) * 1000);
}

function centerText(txt, y) {
	ctx.fillText(txt, (w/2) - (ctx.measureText(txt).width / 2), y);
}

function clearSelection() {
    if(document.selection && document.selection.empty) {
        document.selection.empty();
    } else if(window.getSelection) {
        var sel = window.getSelection();
        sel.removeAllRanges();
    }
}

function strike() {
	var closestCloud = 0;

	for(var i = 0; i < cloudOffsets.length; i++) {
		var co = cloudOffsets[i];
		var cp = {x: co[0] + 23, y: (h/2) + co[1] + 8};
		var mx = {x: mouseX, y: mouseY};
		var cl = cloudOffsets[closestCloud];
		if(dist(cp, mx) < dist({x: cl[0] + 23, y: (h/2) + cl[1] + 8}, mx))
			closestCloud = i;
	}
	
	var cl = cloudOffsets[closestCloud];
	for(var i = 0; i < parseInt(Math.random() * 2) + 1; i++)
		lightnings.push(new Lightning(cl[0] + (Math.random() * 46), (h/2) + cl[1] + 13, mouseX, mouseY));
}

cv.onmousedown = function(e) {
    mouseX = event.pageX - cv.offsetLeft;
    mouseY = event.pageY - cv.offsetTop;
	if(mouseY <= (h / 2) - 20)
		mouseState = MOUSE_STATES.meteor;
	else if(mouseY < (h / 2) + 30) {
		mouseState = MOUSE_STATES.rain;
		//rains.push(new Rain(x - 20, h / 2));
	} else {
		mouseState = MOUSE_STATES.lightning;
		strike();
	}
	
	return false;
	/*var offsetX = Math.random() * 250 - 125;
	rocks.push(new Rock(x - offsetX, -rock.height, (offsetX * 2), 500));*/
};

cv.onmouseup = function(e) {
    mouseX = event.pageX - cv.offsetLeft;
    mouseY = event.pageY - cv.offsetTop;
	
	if(mouseState == MOUSE_STATES.lightning) {
		strike();
	} else if(mouseState == MOUSE_STATES.meteor) {
		var offsetX = Math.random() * 250 - 125;
		rocks.push(new Rock(mouseX - offsetX, -rock.height, (offsetX * 2), 500));
	}
	
	mouseState = MOUSE_STATES.none;
	
	return false;
};

cv.onmousemove = function(e) {
    mouseX = event.pageX - cv.offsetLeft;
    mouseY = event.pageY - cv.offsetTop;
	
	return false;
};

spawnBird();

function draw() {
	now = (new Date()).getTime() / 1000.0;
	delta = (now - lastTime);
	lastTime = now;
	
	requestAnimationFrame(draw);
	
	ctx.clearRect(0, 0, w, h);
	
	var closestCloud = 0;
	
	for(var i = 0; i < cloudOffsets.length; i++) {
		var co = cloudOffsets[i];
		var cp = {x: co[0] + 23, y: (h/2) + co[1] + 8};
		var mx = {x: mouseX, y: mouseY};
		var cl = cloudOffsets[closestCloud];
		if(dist(cp, mx) < dist({x: cl[0] + 23, y: (h/2) + cl[1] + 8}, mx) && mouseState == MOUSE_STATES.rain)
			closestCloud = i;
		
		cloud.draw(co[0], (h / 2) + co[1]);
	}
	
	if(mouseState == MOUSE_STATES.rain) {
		var co = cloudOffsets[closestCloud];
		var cp = {x: co[0] + 23, y: (h/2) + co[1] + 8};
		if(Math.abs(cp.x - mx.x) > 5)
			co[0] += delta * cloudSpeed * 10 * (cp.x > mx.x ? -1 : 1);
		else {
			co[0] = mx.x - 23;
			
			var shouldRain = true;
			for(var i = 0; i < rains.length; i++) {
				var r = rains[i];
				if(r.cloud == closestCloud && r.isRaining) {
					shouldRain = false;
				} else {
					rains[i].isRaining = false;
				}
			}
			
			if(shouldRain) {
				rains.push(new Rain(co[0], cp.y + 5));
			}
		}
		
		cloudOffsets[closestCloud] = co;
	} else {
		for(var i = 0; i < rains.length; i++) {
			rains[i].isRaining = false;
		}
	}
	
	for(var i = 0; i < cloudOffsets.length; i++) {
		var co = cloudOffsets[i];
	
		if(mouseState != MOUSE_STATES.rain || closestCloud != i) {
			co[0] += delta * cloudSpeed;
		}
		
		co[0] = ((co[0] + 46) % (w + 46)) - 46;
		cloudOffsets[i] = co;
	}
	
	//ctx.fillText('swag', 10, 10);
	
	ground.draw(0, h - 15);
	
	for(var i = 0; i < grass.length; i++) {
		grass[i].draw();
	}
	
	if(now >= nextDecision) {
		nextDecision = now + (Math.random() * 4);
		
		if(dinoState == DINO_STATES.idle) {
			if(Math.random() < 0.2) {
				dinoState = DINO_STATES.walking;
			} else if(Math.random() < 0.3) {
				dinoState = DINO_STATES.emote;
				nextDecision++;
			} else {
				if(Math.random() < 0.5) {
					dinoForward = !dinoForward;
				}
			}
		} else if(dinoState == DINO_STATES.walking) {
			dinoState = DINO_STATES.idle;
		} else if(dinoState == DINO_STATES.emote) {
			dinoState = DINO_STATES.idle;
		} else if(dinoState == DINO_STATES.drinking) {
			dinoState = DINO_STATES.emote;
		}
	}
	
	var dinoSprite = dino_idle;
	if((now % 4) < 0.1)
		dinoSprite = dino_blink;
	
	if(dinoState == DINO_STATES.drinking) {
		dinoSprite = now % 0.3 < 0.15 ? dino_step1 : dino_step2;
	} else if(dinoState == DINO_STATES.walking) {
		dinoSprite = now % 0.3 < 0.15 ? dino_step1 : dino_step2;
		
		dinoX += delta * dinoSpeed * (dinoForward ? 1 : -1);
	} else if(dinoState == DINO_STATES.dead) {
		dinoForward = true;
		dinoSprite = dino_dead;
	} else if(dinoState == DINO_STATES.eating) {
		var closestDeadBird = null;
		for(var i = 0; i < birds.length; i++) {
			var bird = birds[i];
			if(bird.dead && bird.eaten < 1) {
				if(closestDeadBird == null)
					closestDeadBird = bird;
				else {
					var p = {x: dinoX, y: h - 47};
					closestDeadBird = dist(p, bird) < dist(closestDeadBird, p) ? bird : closestDeadBird;
				}
			}
		}
		
		if(closestDeadBird == null) {
			dinoState = DINO_STATES.idle;
		} else {
			dinoSprite = now % 0.3 < 0.15 ? dino_step1 : dino_step2;
			dinoForward = closestDeadBird.x > dinoX;
			
			if(dist({x: dinoX, y: closestDeadBird.y}, closestDeadBird) < 32) {
				dinoSprite = now % 0.3 < 0.15 ? dino_crouch2 : dino_crouch1;
				closestDeadBird.eaten += delta;
				if(closestDeadBird.eaten >= 1) {
					dinoStats.hunger += 0.2;
					if(dinoStats.hunger > 0.8) {
						dinoStats.weight += 0.4;
					}
				}
			} else {
				dinoX += delta * (dinoSpeed * 1.5) * (dinoForward ? 1 : -1);
			}
		}
	}
	
	if(dinoStats.hunger < 0.2 && dinoStats.weight > 0) {
		dinoStats.weight -= delta / 60.0;
	} else {
		dinoStats.hunger -= delta / 120.0;
	}
	
	dinoStats.thirst -= delta / 80.0;
	if(dinoStats.thirst < 0 && dinoState != DINO_STATES.dead) {
		dinoState = DINO_STATES.dead;
		deathReason = sel(["he dehydrated", "dont forget to water your dinosaur", "he dried up", "couldnt handle the heat"]);
	}
	
	if(dinoStats.hunger < 0 && dinoState != DINO_STATES.dead) {
		dinoState = DINO_STATES.dead;
		deathReason = sel(["dinosaurs need food too", "dont forget to feed him", "would you look at that", "who wouldve thunk it"]);
	}
	
	if(dinoX < -dino_idle.width) {
		dinoX = w;
	} else if(dinoX > w) {
		dinoX = -dino_idle.width;
	}
	
	for(var i = 0; i < trees.length; i++) {
		trees[i].draw();
	}
	
	for(var i = 0; i < birds.length; i++) {
		birds[i].draw();
	}
	
	if(!dinoForward) {
		ctx.save();
		ctx.scale(-1, 1);
	}
	
	dinoSprite.draw(dinoForward ? parseInt(dinoX) : parseInt(-dino_idle.width - dinoX), h - 47);
	if(dinoSprite != dino_dead) {
		var fatSprites = [null, fat0, fat1, fat2];
		var fatSprite = fatSprites[parseInt(dinoStats.weight)];
		if(fatSprite != null) {
			fatSprite.draw(dinoForward ? parseInt(dinoX) : parseInt(-dino_idle.width - dinoX), h - 47);
		} else if(parseInt(dinoStats.weight) >= fatSprites.length) {
			dinoState = DINO_STATES.dead;
			deathReason = sel(["you overfed him?", "he collapsed into a blackhole", "he exploded"]);
		}
	}
	
	if(!dinoForward) {
		ctx.restore();
	}
	
	if(dinoState == DINO_STATES.emote) {
		var isHappy = dinoStats.hunger > 0.3;
		var emotes = [];
		
		if(dinoStats.hunger <= 0.3) {
			emotes.push(emote_hungry);
		}
		
		if(dinoStats.thirst <= 0.4) {
			emotes.push(emote_thirsty);
		}
			
		if(emotes.length == 0) {
			if(dinoStats.hunger > 0.7 || dinoStats.thirst > 1) {
				emotes.push(emote_full);
			} else {
				emotes.push(emote_content);
			}
		}
		
		var currentEmote = emotes[parseInt((now * 2) % emotes.length)];
		currentEmote.draw(dinoX + (dinoForward ? 20 : 0), h - 47 - 36);
	}
	
	for(var i = 0; i < flowers.length; i++) {
		flowers[i].draw();
	}
	
	for(var i = 0; i < rocks.length; i++) {
		rocks[i].draw();
	}
	
	for(var i = 0; i < rains.length; i++) {
		rains[i].draw();
	}
	
	for(var i = 0; i < lightnings.length; i++) {
		lightnings[i].draw();
	}
	
	if(dinoState == DINO_STATES.dead) {
		ctx.font = "32px 'Segoe UI', Tahoma, sans-serif";
		ctx.fillStyle = "#333";
		centerText("dino is rip", h / 2 - 32);
		centerText(deathReason, h / 2);
	}
}

draw();